@@ -1,9 +1,10 @@
 import numpy
-from typing import List, Dict, Callable, Tuple, Optional, Iterable, Any
+from typing import List, Dict, Callable, Tuple, Optional, Iterable, Any, cast
 from thinc.api import Config, Model, get_current_ops, set_dropout_rate, Ops
 from thinc.api import Optimizer
-from thinc.types import Ragged, Ints2d, Floats2d
+from thinc.types import Ragged, Ints2d, Floats2d, Ints1d
 
+from ..compat import Protocol, runtime_checkable
 from ..scorer import Scorer
 from ..language import Language
 from .trainable_pipe import TrainablePipe
@@ -44,13 +45,19 @@ depth = 4
 DEFAULT_SPANCAT_MODEL = Config().from_str(spancat_default_config)["model"]
 
 
+@runtime_checkable
+class Suggester(Protocol):
+    def __call__(self, docs: Iterable[Doc], *, ops: Optional[Ops] = None) -> Ragged:
+        ...
+
+
 @registry.misc("spacy.ngram_suggester.v1")
-def build_ngram_suggester(sizes: List[int]) -> Callable[[List[Doc]], Ragged]:
+def build_ngram_suggester(sizes: List[int]) -> Suggester:
     """Suggest all spans of the given lengths. Spans are returned as a ragged
     array of integers. The array has two columns, indicating the start and end
     position."""
 
-    def ngram_suggester(docs: List[Doc], *, ops: Optional[Ops] = None) -> Ragged:
+    def ngram_suggester(docs: Iterable[Doc], *, ops: Optional[Ops] = None) -> Ragged:
         if ops is None:
             ops = get_current_ops()
         spans = []
@@ -67,10 +74,11 @@ def build_ngram_suggester(sizes: List[int]) -> Callable[[List[Doc]], Ragged]:
                 if spans:
                     assert spans[-1].ndim == 2, spans[-1].shape
             lengths.append(length)
+        lengths_array = cast(Ints1d, ops.asarray(lengths, dtype="i"))
         if len(spans) > 0:
-            output = Ragged(ops.xp.vstack(spans), ops.asarray(lengths, dtype="i"))
+            output = Ragged(ops.xp.vstack(spans), lengths_array)
         else:
-            output = Ragged(ops.xp.zeros((0, 0)), ops.asarray(lengths, dtype="i"))
+            output = Ragged(ops.xp.zeros((0, 0)), lengths_array)
 
         assert output.dataXd.ndim == 2
         return output
@@ -79,13 +87,11 @@ def build_ngram_suggester(sizes: List[int]) -> Callable[[List[Doc]], Ragged]:
 
 
 @registry.misc("spacy.ngram_range_suggester.v1")
-def build_ngram_range_suggester(
-    min_size: int, max_size: int
-) -> Callable[[List[Doc]], Ragged]:
+def build_ngram_range_suggester(min_size: int, max_size: int) -> Suggester:
     """Suggest all spans of the given lengths between a given min and max value - both inclusive.
     Spans are returned as a ragged array of integers. The array has two columns,
     indicating the start and end position."""
-    sizes = range(min_size, max_size + 1)
+    sizes = list(range(min_size, max_size + 1))
     return build_ngram_suggester(sizes)
 
 
@@ -98,23 +104,25 @@ def build_ngram_range_suggester(
         "max_positive": None,
         "model": DEFAULT_SPANCAT_MODEL,
         "suggester": {"@misc": "spacy.ngram_suggester.v1", "sizes": [1, 2, 3]},
+        "scorer": {"@scorers": "spacy.spancat_scorer.v1"},
     },
     default_score_weights={"spans_sc_f": 1.0, "spans_sc_p": 0.0, "spans_sc_r": 0.0},
 )
 def make_spancat(
     nlp: Language,
     name: str,
-    suggester: Callable[[List[Doc]], Ragged],
+    suggester: Suggester,
     model: Model[Tuple[List[Doc], Ragged], Floats2d],
     spans_key: str,
-    threshold: float = 0.5,
-    max_positive: Optional[int] = None,
+    scorer: Optional[Callable],
+    threshold: float,
+    max_positive: Optional[int],
 ) -> "SpanCategorizer":
     """Create a SpanCategorizer component. The span categorizer consists of two
     parts: a suggester function that proposes candidate spans, and a labeller
     model that predicts one or more labels for each span.
 
-    suggester (Callable[List[Doc], Ragged]): A function that suggests spans.
+    suggester (Callable[[Iterable[Doc], Optional[Ops]], Ragged]): A function that suggests spans.
         Spans are returned as a ragged array with two integer columns, for the
         start and end positions.
     model (Model[Tuple[List[Doc], Ragged], Floats2d]): A model instance that
@@ -138,9 +146,28 @@ def make_spancat(
         threshold=threshold,
         max_positive=max_positive,
         name=name,
+        scorer=scorer,
     )
 
 
+def spancat_score(examples: Iterable[Example], **kwargs) -> Dict[str, Any]:
+    kwargs = dict(kwargs)
+    attr_prefix = "spans_"
+    key = kwargs["spans_key"]
+    kwargs.setdefault("attr", f"{attr_prefix}{key}")
+    kwargs.setdefault("allow_overlap", True)
+    kwargs.setdefault(
+        "getter", lambda doc, key: doc.spans.get(key[len(attr_prefix) :], [])
+    )
+    kwargs.setdefault("has_annotation", lambda doc: key in doc.spans)
+    return Scorer.score_spans(examples, **kwargs)
+
+
+@registry.scorers("spacy.spancat_scorer.v1")
+def make_spancat_scorer():
+    return spancat_score
+
+
 class SpanCategorizer(TrainablePipe):
     """Pipeline component to label spans of text.
 
@@ -151,14 +178,31 @@ class SpanCategorizer(TrainablePipe):
         self,
         vocab: Vocab,
         model: Model[Tuple[List[Doc], Ragged], Floats2d],
-        suggester: Callable[[List[Doc]], Ragged],
+        suggester: Suggester,
         name: str = "spancat",
         *,
         spans_key: str = "spans",
         threshold: float = 0.5,
         max_positive: Optional[int] = None,
+        scorer: Optional[Callable] = spancat_score,
     ) -> None:
         """Initialize the span categorizer.
+        vocab (Vocab): The shared vocabulary.
+        model (thinc.api.Model): The Thinc Model powering the pipeline component.
+        name (str): The component instance name, used to add entries to the
+            losses during training.
+        spans_key (str): Key of the Doc.spans dict to save the spans under.
+            During initialization and training, the component will look for
+            spans on the reference document under the same key. Defaults to
+            `"spans"`.
+        threshold (float): Minimum probability to consider a prediction
+            positive. Spans with a positive prediction will be saved on the Doc.
+            Defaults to 0.5.
+        max_positive (Optional[int]): Maximum number of labels to consider
+            positive per span. Defaults to None, indicating no limit.
+        scorer (Optional[Callable]): The scoring method. Defaults to
+            Scorer.score_spans for the Doc.spans[spans_key] with overlapping
+            spans allowed.
 
         DOCS: https://spacy.io/api/spancategorizer#init
         """
@@ -172,6 +216,7 @@ class SpanCategorizer(TrainablePipe):
         self.suggester = suggester
         self.model = model
         self.name = name
+        self.scorer = scorer
 
     @property
     def key(self) -> str:
@@ -179,7 +224,7 @@ class SpanCategorizer(TrainablePipe):
         initialization and training, the component will look for spans on the
         reference document under the same key.
         """
-        return self.cfg["spans_key"]
+        return str(self.cfg["spans_key"])
 
     def add_label(self, label: str) -> int:
         """Add a new label to the pipe.
@@ -194,7 +239,7 @@ class SpanCategorizer(TrainablePipe):
         if label in self.labels:
             return 0
         self._allow_extra_label()
-        self.cfg["labels"].append(label)
+        self.cfg["labels"].append(label)  # type: ignore
         self.vocab.strings.add(label)
         return 1
 
@@ -204,7 +249,7 @@ class SpanCategorizer(TrainablePipe):
 
         DOCS: https://spacy.io/api/spancategorizer#labels
         """
-        return tuple(self.cfg["labels"])
+        return tuple(self.cfg["labels"])  # type: ignore
 
     @property
     def label_data(self) -> List[str]:
@@ -223,8 +268,8 @@ class SpanCategorizer(TrainablePipe):
         DOCS: https://spacy.io/api/spancategorizer#predict
         """
         indices = self.suggester(docs, ops=self.model.ops)
-        scores = self.model.predict((docs, indices))
-        return (indices, scores)
+        scores = self.model.predict((docs, indices))  # type: ignore
+        return indices, scores
 
     def set_annotations(self, docs: Iterable[Doc], indices_scores) -> None:
         """Modify a batch of Doc objects, using pre-computed scores.
@@ -240,7 +285,7 @@ class SpanCategorizer(TrainablePipe):
         for i, doc in enumerate(docs):
             indices_i = indices[i].dataXd
             doc.spans[self.key] = self._make_span_group(
-                doc, indices_i, scores[offset : offset + indices.lengths[i]], labels
+                doc, indices_i, scores[offset : offset + indices.lengths[i]], labels  # type: ignore[arg-type]
             )
             offset += indices.lengths[i]
 
@@ -279,14 +324,14 @@ class SpanCategorizer(TrainablePipe):
         set_dropout_rate(self.model, drop)
         scores, backprop_scores = self.model.begin_update((docs, spans))
         loss, d_scores = self.get_loss(examples, (spans, scores))
-        backprop_scores(d_scores)
+        backprop_scores(d_scores)  # type: ignore
         if sgd is not None:
             self.finish_update(sgd)
         losses[self.name] += loss
         return losses
 
     def get_loss(
-        self, examples: Iterable[Example], spans_scores: Tuple[Ragged, Ragged]
+        self, examples: Iterable[Example], spans_scores: Tuple[Ragged, Floats2d]
     ) -> Tuple[float, float]:
         """Find the loss and gradient of loss for the batch of documents and
         their predicted scores.
@@ -311,8 +356,8 @@ class SpanCategorizer(TrainablePipe):
             spans_index = {}
             spans_i = spans[i].dataXd
             for j in range(spans.lengths[i]):
-                start = int(spans_i[j, 0])
-                end = int(spans_i[j, 1])
+                start = int(spans_i[j, 0])  # type: ignore
+                end = int(spans_i[j, 1])  # type: ignore
                 spans_index[(start, end)] = offset + j
             for gold_span in self._get_aligned_spans(eg):
                 key = (gold_span.start, gold_span.end)
@@ -323,7 +368,7 @@ class SpanCategorizer(TrainablePipe):
             # The target is a flat array for all docs. Track the position
             # we're at within the flat array.
             offset += spans.lengths[i]
-        target = self.model.ops.asarray(target, dtype="f")
+        target = self.model.ops.asarray(target, dtype="f")  # type: ignore
         # The target will have the values 0 (for untrue predictions) or 1
         # (for true predictions).
         # The scores should be in the range [0, 1].
@@ -339,7 +384,7 @@ class SpanCategorizer(TrainablePipe):
         self,
         get_examples: Callable[[], Iterable[Example]],
         *,
-        nlp: Language = None,
+        nlp: Optional[Language] = None,
         labels: Optional[List[str]] = None,
     ) -> None:
         """Initialize the pipe for training, using a representative set
@@ -347,14 +392,14 @@ class SpanCategorizer(TrainablePipe):
 
         get_examples (Callable[[], Iterable[Example]]): Function that
             returns a representative sample of gold-standard Example objects.
-        nlp (Language): The current nlp object the component is part of.
-        labels: The labels to add to the component, typically generated by the
+        nlp (Optional[Language]): The current nlp object the component is part of.
+        labels (Optional[List[str]]): The labels to add to the component, typically generated by the
             `init labels` command. If no labels are provided, the get_examples
             callback is used to extract the labels from the data.
 
         DOCS: https://spacy.io/api/spancategorizer#initialize
         """
-        subbatch = []
+        subbatch: List[Example] = []
         if labels is not None:
             for label in labels:
                 self.add_label(label)
@@ -373,32 +418,14 @@ class SpanCategorizer(TrainablePipe):
         else:
             self.model.initialize()
 
-    def score(self, examples: Iterable[Example], **kwargs) -> Dict[str, Any]:
-        """Score a batch of examples.
-
-        examples (Iterable[Example]): The examples to score.
-        RETURNS (Dict[str, Any]): The scores, produced by Scorer.score_cats.
-
-        DOCS: https://spacy.io/api/spancategorizer#score
-        """
-        validate_examples(examples, "SpanCategorizer.score")
-        self._validate_categories(examples)
-        kwargs = dict(kwargs)
-        attr_prefix = "spans_"
-        kwargs.setdefault("attr", f"{attr_prefix}{self.key}")
-        kwargs.setdefault("allow_overlap", True)
-        kwargs.setdefault(
-            "getter", lambda doc, key: doc.spans.get(key[len(attr_prefix) :], [])
-        )
-        kwargs.setdefault("has_annotation", lambda doc: self.key in doc.spans)
-        return Scorer.score_spans(examples, **kwargs)
-
-    def _validate_categories(self, examples):
+    def _validate_categories(self, examples: Iterable[Example]):
         # TODO
         pass
 
     def _get_aligned_spans(self, eg: Example):
-        return eg.get_aligned_spans_y2x(eg.reference.spans.get(self.key, []), allow_overlap=True)
+        return eg.get_aligned_spans_y2x(
+            eg.reference.spans.get(self.key, []), allow_overlap=True
+        )
 
     def _make_span_group(
         self, doc: Doc, indices: Ints2d, scores: Floats2d, labels: List[str]
@@ -408,10 +435,11 @@ class SpanCategorizer(TrainablePipe):
         threshold = self.cfg["threshold"]
 
         keeps = scores >= threshold
-        ranked = (scores * -1).argsort()
+        ranked = (scores * -1).argsort()  # type: ignore
         if max_positive is not None:
-            filter = ranked[:, max_positive:]
-            for i, row in enumerate(filter):
+            assert isinstance(max_positive, int)
+            span_filter = ranked[:, max_positive:]
+            for i, row in enumerate(span_filter):
                 keeps[i, row] = False
         spans.attrs["scores"] = scores[keeps].flatten()
 
